# Generated by Django 3.2.12 on 2022-04-11 10:59

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("emeis_core", "0007_change_JSONField_to_native_django_model_field"),
    ]

    operations = [
        migrations.AddField(
            model_name="scope",
            name="is_active",
            field=models.BooleanField(default=True),
        ),
    ]
