# Generated by Django 3.2.12 on 2022-04-21 07:24

from django.db import migrations
import django.db.models.deletion
import mptt.fields


class Migration(migrations.Migration):
    dependencies = [
        ("emeis_core", "0008_scope_is_active"),
    ]

    operations = [
        migrations.AlterField(
            model_name="scope",
            name="parent",
            field=mptt.fields.TreeForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="children",
                to="emeis_core.scope",
            ),
        ),
    ]
