# Generated by Django 2.2.13 on 2020-06-18 07:55

import localized_fields.fields.char_field
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("emeis_core", "0002_data_bootstrap"),
    ]

    operations = [
        migrations.AlterField(
            model_name="user",
            name="city",
            field=localized_fields.fields.char_field.LocalizedCharField(
                blank=True, max_length=255, null=True, required=[], verbose_name="city"
            ),
        ),
    ]
